@@ -1,42 +1,12 @@
-//**********************************************************************;
-// Copyright (c) 2017, Intel Corporation
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice,
-// this list of conditions and the following disclaimer.
-//
-// 2. Redistributions in binary form must reproduce the above copyright notice,
-// this list of conditions and the following disclaimer in the documentation
-// and/or other materials provided with the distribution.
-//
-// 3. Neither the name of Intel Corporation nor the names of its contributors
-// may be used to endorse or promote products derived from this software without
-// specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
-// THE POSSIBILITY OF SUCH DAMAGE.
-//**********************************************************************;
+/* SPDX-License-Identifier: BSD-3-Clause */
+
 #include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <tss2/tss2_sys.h>
-
 #include "log.h"
 #include "tpm2_attr_util.h"
-#include "tpm2_util.h"
 
 #define dispatch_no_arg_add(x) \
     { .name = str(x), .callback=(action)x, .width = 1 }
@@ -49,9 +19,7 @@
 
 typedef enum dispatch_error dispatch_error;
 enum dispatch_error {
-    dispatch_ok = 0,
-    dispatch_err,
-    dispatch_no_match,
+    dispatch_ok = 0, dispatch_err, dispatch_no_match,
 };
 
 typedef bool (*action)(void *obj, char *arg);
@@ -387,7 +355,8 @@ static dispatch_table obj_attr_table[] = {         // Bit Index
         dispatch_reserved(31),                     // 31
 };
 
-static bool token_match(const char *name, const char *token, bool has_arg, char **sep) {
+static bool token_match(const char *name, const char *token, bool has_arg,
+        char **sep) {
 
     /* if it has an argument, we expect a separator */
     size_t match_len = strlen(token);
@@ -420,7 +389,7 @@ static dispatch_error handle_dispatch(dispatch_table *d, char *token,
     }
 
     /*
-     * If it has an argument, match should have found the seperator.
+     * If it has an argument, match should have found the separator.
      */
     char *arg = NULL;
     if (has_arg) {
@@ -445,13 +414,14 @@ static dispatch_error handle_dispatch(dispatch_table *d, char *token,
     return result ? dispatch_ok : dispatch_err;
 }
 
-static bool common_strtoattr(char *attribute_list, void *attrs, dispatch_table *table, size_t size) {
+static bool common_strtoattr(char *attribute_list, void *attrs,
+        dispatch_table *table, size_t size) {
 
     char *token;
     char *save;
 
     /*
-     * This check is soley to prevent GCC from complaining on:
+     * This check is solely to prevent GCC from complaining on:
      * error: ‘attribute_list’ may be used uninitialized in this function [-Werror=maybe-uninitialized]
      * Might as well check nvattrs as well.
      */
@@ -496,15 +466,17 @@ static bool common_strtoattr(char *attribute_list, void *attrs, dispatch_table *
 bool tpm2_attr_util_nv_strtoattr(char *attribute_list, TPMA_NV *nvattrs) {
 
     memset(nvattrs, 0, sizeof(*nvattrs));
-    return common_strtoattr(attribute_list, nvattrs, nv_attr_table, ARRAY_LEN(nv_attr_table));
+    return common_strtoattr(attribute_list, nvattrs, nv_attr_table,
+            ARRAY_LEN(nv_attr_table));
 }
 
 bool tpm2_attr_util_obj_strtoattr(char *attribute_list, TPMA_OBJECT *objattrs) {
 
-    return common_strtoattr(attribute_list, objattrs, obj_attr_table, ARRAY_LEN(obj_attr_table));
+    memset(objattrs, 0, sizeof(*objattrs));
+    return common_strtoattr(attribute_list, objattrs, obj_attr_table,
+            ARRAY_LEN(obj_attr_table));
 }
 
-
 static UINT8 find_first_set(UINT32 bits) {
 
     UINT8 n = 0;
@@ -513,16 +485,30 @@ static UINT8 find_first_set(UINT32 bits) {
         return n;
     }
 
-    if (!(bits & 0x0000FFFF)) { n += 16; bits >>= 16; }
-    if (!(bits & 0x000000FF)) { n +=  8; bits >>=  8; }
-    if (!(bits & 0x0000000F)) { n +=  4; bits >>=  4; }
-    if (!(bits & 0x00000003)) { n +=  2; bits >>=  2; }
-    if (!(bits & 0x00000001))   n +=  1;
+    if (!(bits & 0x0000FFFF)) {
+        n += 16;
+        bits >>= 16;
+    }
+    if (!(bits & 0x000000FF)) {
+        n += 8;
+        bits >>= 8;
+    }
+    if (!(bits & 0x0000000F)) {
+        n += 4;
+        bits >>= 4;
+    }
+    if (!(bits & 0x00000003)) {
+        n += 2;
+        bits >>= 2;
+    }
+    if (!(bits & 0x00000001))
+        n += 1;
 
     return n;
 }
 
-static char *tpm2_attr_util_common_attrtostr(UINT32 attrs, dispatch_table *table, size_t size) {
+static char *tpm2_attr_util_common_attrtostr(UINT32 attrs,
+        dispatch_table *table, size_t size) {
 
     if (attrs == 0) {
         return strdup("<none>");
@@ -542,7 +528,7 @@ static char *tpm2_attr_util_common_attrtostr(UINT32 attrs, dispatch_table *table
 
     size_t i;
     size_t max_name_len = 0;
-    for (i=0; i < size; i++) {
+    for (i = 0; i < size; i++) {
         dispatch_table *d = &table[i];
         size_t name_len = strlen(d->name);
         max_name_len = name_len > max_name_len ? name_len : max_name_len;
@@ -555,7 +541,6 @@ static char *tpm2_attr_util_common_attrtostr(UINT32 attrs, dispatch_table *table
         return NULL;
     }
 
-
     size_t string_index = 0;
 
     /*
@@ -575,7 +560,7 @@ static char *tpm2_attr_util_common_attrtostr(UINT32 attrs, dispatch_table *table
         size_t left = length - string_index;
 
         /* this is a mask that is field width wide */
-        UINT8 mask = ((UINT32)1 << w) - 1;
+        UINT8 mask = ((UINT32) 1 << w) - 1;
 
         /* get the value in the field before clearing attrs out */
         UINT8 field_values = (attrs & mask << bit_index) >> bit_index;
@@ -586,6 +571,7 @@ static char *tpm2_attr_util_common_attrtostr(UINT32 attrs, dispatch_table *table
          */
         attrs &= ~(mask << bit_index);
 
+        int n;
         /*
          * if the callback is NULL, we are either in a field middle or reserved
          * section which is weird, just add the name in. In the case of being
@@ -596,43 +582,31 @@ static char *tpm2_attr_util_common_attrtostr(UINT32 attrs, dispatch_table *table
          * Fields are either 1 or > 1.
          */
         if (w == 1) {
-            /*
-             * set the format to a middle output, unless we're parsing
-             * the first or last. Let the format be static with the routine
-             * so the compiler can do printf style format specifier checking.
-             */
-            if (!string_index) {
-                /* on the first write, if we are already done, no pipes */
-                string_index += !attrs ? snprintf(s, left, "%s", name) :
-                        snprintf(s, left, "%s|", name);
-            } else if (!attrs) {
-                string_index += snprintf(s, left, "%s", name);
-            } else {
-                string_index += snprintf(s, left, "%s|", name);
-            }
+            n = snprintf(s, left, attrs ? "%s|" : "%s", name);
         } else {
             /* deal with the field */
-            if (!string_index) {
-                /* on the first write, if we are already done, no pipes */
-                string_index += !attrs ? snprintf(s, left, "%s=0x%X", name, field_values) :
-                        snprintf(s, left, "%s=0x%X|", name, field_values);
-            } else if (!attrs) {
-                string_index += snprintf(s, left, "%s=0x%X", name, field_values);
-            } else {
-                string_index += snprintf(s, left, "%s=0x%X|", name, field_values);
-            }
+            n = snprintf(s, left, attrs ? "%s=0x%X|" : "%s=0x%X", name,
+                    field_values);
         }
+
+        /* check if there was enough space left in s */
+        if ((n < 0) || ((unsigned) n >= left)) {
+            break;
+        }
+        string_index += n;
     }
 
     return str;
 }
 
 char *tpm2_attr_util_nv_attrtostr(TPMA_NV nvattrs) {
-    return tpm2_attr_util_common_attrtostr(nvattrs, nv_attr_table, ARRAY_LEN(nv_attr_table));
+    return tpm2_attr_util_common_attrtostr(nvattrs, nv_attr_table,
+            ARRAY_LEN(nv_attr_table));
 }
 
 char *tpm2_attr_util_obj_attrtostr(TPMA_OBJECT objattrs) {
-    return tpm2_attr_util_common_attrtostr(objattrs, obj_attr_table, ARRAY_LEN(obj_attr_table));
+    return tpm2_attr_util_common_attrtostr(objattrs, obj_attr_table,
+            ARRAY_LEN(obj_attr_table));
 }
 
 bool tpm2_attr_util_obj_from_optarg(char *argvalue, TPMA_OBJECT *objattrs) {
